@@ -1,23 +1,45 @@
 #!/usr/bin/perl
 
+# Copyright [2015-2018] EMBL-European Bioinformatics Institute
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 use strict;
 use File::Temp 'tempdir';
 use Cwd;
 use File::Path qw(make_path);
 use Getopt::Long;
 
+my $htslib_version = "1.5";
+my $version = get_version();
+
 my $help = "INSTALL.pl [-h|--help] [--prefix=filepath] [--static] [~/prefix/path]\n";
 $help .= "--help (-h)  - this help message\n";
 $help .= "--prefix     - Path to install Bio::DB::HTS.\n";
 $help .= "                  Alternative to providing a path at the end of comandline\n";
 $help .= "                  If neither are provided defaults to root install (requires root access)\n";
 $help .= "--static     - Build Bio::DB::HTS using the static libhts.a library and do not install htslib\n";
+$help .= "--htslib_version     - Build Bio::DB::HTS using the specified HTSlib version (Default ".$htslib_version.")\n";
 
 my $cwd = system 'pwd';
 
+
 my $opts = parse_options();
 my $prefix_path;
 $prefix_path = $opts->{'prefix'} if(exists($opts->{'prefix'}) && defined($opts->{'prefix'}));
+$htslib_version = $opts->{'htslib_version'} if($opts->{'htslib_version'}) ;
+
+my ($version_major,$version_minor,$version_revision) = split /\./, $htslib_version ;
 
 
 # STEP 0: various dependencies
@@ -44,13 +66,35 @@ On Debian/Ubuntu systems you can do this with the command:
   apt-get install build-essential
 END
 
+# The following libraries are version number dependant for HTSlib
+if( $version_major >= 2 || ($version_major==1 && $version_minor>=5) )
+{
+-e '/usr/include/lzma.h' or die <<END;
+lzma.h library header not found in /usr/include. Please install it and try again.
+On Debian/Ubuntu systems you can do this with the command:
+
+  apt-get install liblzma-dev
+END
+
+-e '/usr/include/bzlib.h' or die <<END;
+zlib.h library header not found in /usr/include. Please install it and try again.
+On Debian/Ubuntu systems you can do this with the command:
+
+  apt-get install libbz2-dev
+END
+}
+else
+{
 -e '/usr/include/zlib.h' or die <<END;
 zlib.h library header not found in /usr/include. Please install it and try again.
 On Debian/Ubuntu systems you can do this with the command:
 
   apt-get install zlib1g-dev
 END
+
     ;
+}
+
 
 eval "require Bio::SeqFeature::Lite" or die <<END;
 BioPerl does not seem to be installed. Please install it and try again.
@@ -69,20 +113,36 @@ my $install_dir = tempdir(CLEANUP => 1);
 info("Performing build in $install_dir");
 
 
-# STEP 2: Check out HTSlib
-info("Checking out HTSlib");
+# STEP 2: Download HTSlib
+info("Checking out HTSlib v".$htslib_version);
 chdir $install_dir;
-system "git clone -b master --depth=1 https://github.com/samtools/htslib.git";
--d './htslib' or die "git clone seems to have failed. Could not find $install_dir/htslib directory";
-chdir './htslib';
+my $htslib_archive = $htslib_version.".zip" ;
+my $htslib_archive_url = "https://github.com/samtools/htslib/archive/".$htslib_version.".zip" ;
+system "wget -O " . $htslib_archive . " " .$htslib_archive_url  ;
+if ( $? == -1 )
+{
+  die "HTSlib fetch command failed: $!\n";
+}
+-f './'.$htslib_archive or die "Could not fetch HTSlib archive ".$htslib_archive_url ;
+system "unzip ".$htslib_archive ;
+system "mv htslib-$htslib_version htslib" ;
+-d './htslib' or die "Unzip seems to have failed. Could not find $install_dir/htslib directory";
 
 
-# STEP 3: Check out Bio-HTS
-info("Checking out Bio-HTS");
+# STEP 3: Download Bio-DB-HTS
+info("Fetching version $version of Bio-DB-HTS from GitHub");
 chdir $install_dir;
-system "git clone -b master --depth=1 https://github.com/Ensembl/Bio-HTS.git";
--d './Bio-HTS' or die "git clone seems to have failed. Could not find $install_dir/Bio-HTS directory";
-chdir "./Bio-HTS";
+my $biodbhts_archive = "$version.zip" ;
+my $biodbhts_archive_url = "https://github.com/Ensembl/Bio-DB-HTS/archive/$version.zip" ;
+system "wget -O " . $biodbhts_archive . " " .$biodbhts_archive_url  ;
+if ( $? == -1 )
+{
+  die "Bio-DB-HTS fetch command failed: $!\n";
+}
+-f './'.$biodbhts_archive or die "Could not fetch Bio::DB::HTS archive ".$biodbhts_archive_url ;
+system "unzip ".$biodbhts_archive ;
+system "mv Bio-DB-HTS-$version Bio-DB-HTS" ;
+-d './Bio-DB-HTS' or die "Unzip seems to have failed. Could not find $install_dir/Bio-DB-HTS directory";
 
 
 # Step 4: Build libhts.a
@@ -115,9 +175,9 @@ if($opts->{'static'}){
 }
 -e 'libhts.a' or die "Compile didn't complete. No libhts.a library file found";
 
-# Step 5: Build Bio::DB::HTSlib
-info("Building Bio::DB::HTSlib");
-chdir "$install_dir/Bio-HTS";
+# Step 5: Build Bio::DB::HTS
+info("Building Bio::DB::HTS");
+chdir "$install_dir/Bio-DB-HTS";
 my $cmd;
 if(defined $prefix_path) {
   if(!$opts->{'static'}){
@@ -152,7 +212,7 @@ if($opts->{'static'}){
 }
 
 # Step 7: Yay!
-info("Bio::DB::HTS is now installed.");
+info("Bio::DB::HTS v$version is now installed.");
 chdir '/';
 
 exit 0;
@@ -164,6 +224,7 @@ sub parse_options {
   my $result = &GetOptions (
     'static' => \$opts{'static'},
     'prefix=s' => \$opts{'prefix'},
+    'htslib_version=s' => \$opts{'htslib_version'},
     'h|help' => \$opts{'h'},
   );
 
@@ -216,3 +277,16 @@ sub prefix_install {
   }
   return $prefix_path;
 }
+
+sub get_version {
+  use FindBin '$Bin';
+  my $version = `grep -m 1 "HTS::VERSION" $Bin/lib/Bio/DB/HTS.pm`;
+  $version =~ /(\d+?\.\d+?)\D/;
+  $version = $1;
+
+  use Carp;
+  croak "Couldn't get a meaningful library version: $version"
+    unless $version =~ /\d+?\.\d+?/;
+
+  return $version;
+}
