@@ -1,7 +1,7 @@
 
 =head1 LICENSE
 
-Copyright [1999-2016] EMBL-European Bioinformatics Institute
+Copyright [2015-2018] EMBL-European Bioinformatics Institute
 
 
 Licensed under the Apache License, Version 2.0 (the "License");
@@ -55,7 +55,7 @@ A hashref that maps flag values to human-readable names. For example:
 
 The reverse of FLAGS:
 
- FLAGS->{M_UNMAPPED} == 0x0008
+ RFLAGS->{M_UNMAPPED} == 0x0008
 
 =back
 
@@ -70,7 +70,7 @@ L<Bio::Perl>, L<Bio::DB::HTS>, L<Bio::DB::Bam::Alignment>
 =cut
 
 package Bio::DB::HTS::Constants;
-$Bio::DB::HTS::Constants::VERSION = '2.1';
+$Bio::DB::HTS::Constants::VERSION = '2.11';
 
 use strict;
 use warnings;
